@@ -18,7 +18,7 @@
 #include "ldastoolsal/MemChecker.hh"
 #include "ldastoolsal/Directory.hh"
 #include "ldastoolsal/mutexlock.hh"
-#include "ldastoolsal/Pool.hh"
+#include "ldastoolsal/TriStatePool.hh"
 #include "ldastoolsal/SharedPtr.hh"
 #include "ldastoolsal/ErrorLog.hh"
 #include "ldastoolsal/Singleton.hh"
@@ -30,6 +30,9 @@ using LDASTools::AL::ErrorLog;
 using LDASTools::AL::StdErrLog;
 using LDASTools::System::ErrnoMessage;
 
+#define DYNAMIC_BUFFER 1
+#define READDIR_R_DEPRICATED 1
+
 #ifndef NAME_MAX
 #ifdef FILENAME_MAX
 #define NAME_MAX FILENAME_MAX
@@ -38,7 +41,7 @@ using LDASTools::System::ErrnoMessage;
 
 typedef struct dirent dirent_type;
 using LDASTools::AL::Directory;
-using LDASTools::AL::Pool;
+using LDASTools::AL::TriStatePool;
 using LDASTools::AL::SystemCallTask;
 using LDASTools::AL::TIMEOUT_COMPLETED;
 using LDASTools::AL::Timeout;
@@ -49,7 +52,8 @@ namespace LDASTools
   namespace AL
   {
     class Directory::Internals
-      : public SystemCallTask
+      : public SystemCallTask,
+	public TriStateInterface
     {
     public:
       static int		m_wait_time_max;
@@ -78,7 +82,7 @@ namespace LDASTools
 
       Internals( );
 
-      ~Internals( );
+      virtual ~Internals( );
  
       //-----------------------------------------------------------------
       /// \brief Store the name of the directory to be read.
@@ -114,6 +118,8 @@ namespace LDASTools
 
       int Eval( mode_type Mode );
 
+      virtual bool makeAvailable( ) const;
+
     protected:
       //-----------------------------------------------------------------
       /// \brief Do the system call
@@ -124,13 +130,22 @@ namespace LDASTools
       ///     is set.
       //-----------------------------------------------------------------
       virtual int eval( );
+      state_type stateNoLock( ) const;
+      void stateNoLock( state_type State );
 
     private:
-      std::string				m_dirname;
+      std::string			m_dirname;
       DIR*				m_dirp;
-      int					m_fd;
+      int				m_fd;
+#if DYNAMIC_BUFFER
       dirent_type*			m_buffer;
-      state_type				m_state;
+#else /* DYNAMIC_BUFFER */
+      union {
+	dirent_type			entry;
+	char                            padding[8 * 1024];
+      } m_buffer;
+#endif /* DYNAMIC_BUFFER */
+      state_type			m_state;
       mutable MutexLock::baton_type	m_state_baton;
 
       void mode( mode_type Mode );
@@ -155,7 +170,11 @@ namespace LDASTools
     inline const char* Directory::Internals::
     Filename( ) const
     {
+#if DYNAMIC_BUFFER
       return m_buffer->d_name;
+#else /* DYNAMIC_BUFFER */
+      return m_buffer.entry.d_name;
+#endif /* DYNAMIC_BUFFER */
     }
 
     inline Directory::Internals::state_type Directory::Internals::
@@ -164,7 +183,7 @@ namespace LDASTools
       MutexLock	l( m_state_baton,
 		   __FILE__, __LINE__ );
 
-      return m_state;
+      return stateNoLock( );
     }
 
     inline void Directory::Internals::
@@ -173,6 +192,18 @@ namespace LDASTools
       MutexLock	l( m_state_baton,
 		   __FILE__, __LINE__ );
 
+      stateNoLock( State );
+    }
+
+    inline Directory::Internals::state_type Directory::Internals::
+    stateNoLock( ) const
+    {
+      return m_state;
+    }
+
+    inline void Directory::Internals::
+    stateNoLock( Directory::Internals::state_type State )
+    {
       m_state = State;
     }
 
@@ -203,6 +234,12 @@ namespace LDASTools
       return m_internals->DirName( );
     }
 
+    int Directory::
+    Timeout( )
+    {
+      return Internals::m_wait_time_max;
+    }
+
     void Directory::
     Timeout( int Value )
     {
@@ -219,11 +256,12 @@ namespace
   /// \brief Collection of reusable buffers
   //---------------------------------------------------------------------
   class DirectoryPool
-    : public Pool< value_type >
+    : public TriStatePool< value_type >
   {
     SINGLETON_TS_DECL( DirectoryPool );
   };
 
+#if 0
   //---------------------------------------------------------------------
   /// \brief Collection of buffers that have timed out
   ///
@@ -240,6 +278,7 @@ namespace
   {
     SINGLETON_TS_DECL( DirectoryTrashPool );
   };
+#endif /* 0 */
 
   //---------------------------------------------------------------------
   /// \brief Create a new instance for use in the pool
@@ -273,7 +312,9 @@ namespace LDASTools
       : m_dirname( "" ),
 	m_dirp( (DIR*)NULL ),
 	m_fd( -1 ),
+#if DYNAMIC_BUFFER
 	m_buffer( (dirent_type*)NULL ),
+#endif /* DYNAMIC_BUFFER */
 	m_state( STATE_CLOSED ),
 	m_state_baton( __FILE__, __LINE__ )
     {
@@ -316,7 +357,7 @@ namespace LDASTools
 	    MutexLock	l( m_state_baton,
 			   __FILE__, __LINE__ );
 
-	    if ( ( m_state == STATE_OPEN )
+	    if ( ( stateNoLock( ) == STATE_OPEN )
 		 && ( m_dirp ) )
 	    {
 	      do_it = true;
@@ -338,12 +379,20 @@ namespace LDASTools
 	    dirent_type*	result = (dirent_type*)NULL;
 	    while( true )
 	    {
+#if READDIR_R_DEPRICATED
+	      result = readdir( m_dirp );
+#else /* READDIR_R_DEPRICATED */
+#if DYNAMIC_BUFFER
 	      retval = readdir_r( m_dirp, m_buffer, &result );
+#else /* DYNAMIC_BUFFER */
+	      retval = readdir_r( m_dirp, &(m_buffer.entry), &result );
+#endif /* DYNAMIC_BUFFER */
+#endif /* READDIR_R_DEPRICATED */
 	      if ( result == (dirent_type*)NULL )
 	      {
 		break;
 	      }
-	      m_block_read.push_back( Filename( ) );
+	      m_block_read.push_back( result->d_name );
 	    }
 	    {
 	      //-----------------------------------------------------------
@@ -413,11 +462,6 @@ namespace LDASTools
 		{
 		  m_dirp = (DIR*)NULL;
 		  m_fd = -1;
-		  if ( m_buffer )
-		  {
-		    free( m_buffer );
-		    m_buffer = (dirent_type*)NULL;
-		  }
 		}
 	      }
 	    }
@@ -449,7 +493,16 @@ namespace LDASTools
 	    // Read the next entry from the directory
 	    //-------------------------------------------------------------
 	    dirent_type*	result;
+
+#if READDIR_R_DEPRICATED
+	    result = readdir( m_dirp );
+#else /* READDIR_R_DEPRICATED */
+#if DYNAMIC_BUFFER
 	    retval = readdir_r( m_dirp, m_buffer, &result );
+#else /* DYNAMIC_BUFFER */
+	    retval = readdir_r( m_dirp, &(m_buffer.entry), &result );
+#endif /* DYNAMIC_BUFFER */
+#endif /* READDIR_R_DEPRICATED */
 	    m_retval.next = ( ( retval == 0 ) && ( result != (dirent_type*)NULL ) );
 	    {
 	      //-----------------------------------------------------------
@@ -530,6 +583,7 @@ namespace LDASTools
 	    rewinddir( m_dirp );
 	  }
 	  // Allocate the buffer only once.
+#if DYNAMIC_BUFFER
 	  if ( ! m_buffer )
 	  {
 	    //-------------------------------------------------------------
@@ -563,12 +617,17 @@ namespace LDASTools
 #   endif
 	    size_t struct_size
 	      = (size_t)offsetof(dirent_type, d_name) + name_max + 1;
+	    if ( struct_size < sizeof( *m_buffer ) )
+	    {
+	      struct_size = sizeof( *m_buffer );
+	    }
 	    //-------------------------------------------------------------
 	    // After finding the size of the buffer, it is time to allocate
 	    //   the memory for the buffer for reading directory information.
 	    //-------------------------------------------------------------
 	    m_buffer = (dirent_type*)malloc( struct_size );
 	  }
+#endif /* DYNAMIC_BUFFER */
 	  State( STATE_OPEN );
 	}
 	break;
@@ -579,6 +638,16 @@ namespace LDASTools
       return retval;
     }
 
+    bool Directory::Internals::
+    makeAvailable( ) const
+    {
+      if ( State( ) == STATE_OPEN )
+      {
+	const_cast<Internals *>( this )->Eval( MODE_CLOSE );
+      }
+      return( State( ) == STATE_CLOSED );
+    }
+
     //=====================================================================
     // D I R E C T O R Y
     //=====================================================================
@@ -623,6 +692,7 @@ namespace LDASTools
     {
       if ( m_internals )
       {
+#if 0
 	if ( m_internals->State( ) == Internals::STATE_CLOSED )
 	{
 	  DirectoryPool::Instance( ).Relinquish( m_internals );
@@ -631,6 +701,9 @@ namespace LDASTools
 	{
 	  DirectoryTrashPool::Instance( ).Relinquish( m_internals );
 	}
+#else /* 0 */
+	DirectoryPool::Instance( ).Relinquish( m_internals );
+#endif /* 0 */
       }
     }
 
@@ -685,6 +758,7 @@ namespace
 			5 );
   }
 
+#if 0
   //---------------------------------------------------------------------
   //---------------------------------------------------------------------
   SINGLETON_TS_INST( DirectoryTrashPool );
@@ -696,6 +770,7 @@ namespace
 			"<anonymous>::DirectoryTrashPool::singleton_suicide",
 			5 );
   }
+#endif /* 0 */
 
   int
   process_request( value_type DirInfo )
