@@ -35,6 +35,30 @@ using LDASTools::AL::ReadWriteLock;
 using LDASTools::AL::SignalHandler;
 using LDASTools::AL::unordered_map;
 
+#define LOCAL_VERBOSE_DEBUGGING 0
+
+#if LOCAL_VERBOSE_DEBUGGING > 0
+#define VERBOSE_DEBUGGING(b) std::cerr << b
+#else
+#define VERBOSE_DEBUGGING(b)
+#endif
+
+#if LOCAL_VERBOSE_DEBUGGING >= 10
+#define VERBOSE_DEBUGGING_10( b ) VERBOSE_DEBUGGING(b)
+#else
+#define VERBOSE_DEBUGGING_10( b )
+#endif
+#if LOCAL_VERBOSE_DEBUGGING >= 20
+#define VERBOSE_DEBUGGING_20( b ) VERBOSE_DEBUGGING(b)
+#else
+#define VERBOSE_DEBUGGING_20( b )
+#endif
+#if LOCAL_VERBOSE_DEBUGGING >= 30
+#define VERBOSE_DEBUGGING_30( b ) VERBOSE_DEBUGGING(b)
+#else
+#define VERBOSE_DEBUGGING_30( b )
+#endif
+
 namespace {
   static size_t				stack_size = PTHREAD_STACK_MIN;
 
@@ -460,9 +484,9 @@ namespace LDASTools
 	    //---------------------------------------------------------
 	    // Make sure the signal handler is registered
 	    //---------------------------------------------------------
+	    SignalHandler::ThreadCaptureSignal( m_cancel_via_signal );
 	    SignalHandler::Register( &wakeup,
 				     m_cancel_via_signal );
-	    SignalHandler::ThreadCaptureSignal( m_cancel_via_signal );
 	  }
 	  break;
 	case CANCEL_ASYNCHRONOUS:
@@ -485,11 +509,11 @@ namespace LDASTools
 	  if ( m_cancel_via_signal > 0 )
 	  {
 	    //-------------------------------------------------------------
-	    // Make sure the signal handler is registered
+	    // Make sure the signal handler is unregistered
 	    //-------------------------------------------------------------
-	    SignalHandler::Register( &wakeup,
-				     m_cancel_via_signal );
-	    SignalHandler::ThreadCaptureSignal( m_cancel_via_signal );
+	    SignalHandler::ThreadIgnoreSignal( m_cancel_via_signal );
+	    SignalHandler::Unregister( &wakeup,
+				       m_cancel_via_signal );
 	  }
 	  break;
 	case CANCEL_ASYNCHRONOUS:
@@ -762,10 +786,16 @@ namespace LDASTools
     spawn( start_function_type StartFunction )
     {
       int retval = 0;
+
+      VERBOSE_DEBUGGING_10( "spawn called with p->tid: "  << (unsigned long)( p->tid )
+			    << " IsDetached( ): " << IsDetached( )
+			    << std::endl );
       if ( ( ! p->tid )
 	   || IsDetached( ) )
       {
-
+	VERBOSE_DEBUGGING_10( "Creating new thread with stack size: "
+			      << StackSizeDefault( )
+			      << std::endl );
 	{
 	  MutexLock	l( p->key, __FILE__, __LINE__ );
 
@@ -789,6 +819,8 @@ namespace LDASTools
 	  throw std::runtime_error( "Thread::Start: failed to start thread" );
 	}
       }
+      VERBOSE_DEBUGGING_10( "Completed spawn with retval: " << retval
+			    << std::endl );
       return retval;
     }
 
@@ -805,13 +837,17 @@ namespace LDASTools
 	  {
 	    ThreadSource->CancellationType( CANCEL_EXCEPTION );
 	    ThreadSource->CancellationEnable( true );
+	    VERBOSE_DEBUGGING_30( "start_routine: before action"
+				  << std::endl );
 	    ThreadSource->action( );
-#if 0
-	    throw std::runtime_error( "Bad action" );
-#endif /* 0 */
+	    VERBOSE_DEBUGGING_30( "start_routine: after action"
+				  << std::endl );
 	  }
 	  catch( const std::exception& exception )
 	  {
+	    VERBOSE_DEBUGGING_30( "start_routine: caught exception: "
+				  << exception.what( )
+				  << std::endl );
 	    if ( ThreadSource->IsCancelled( ) )
 	    {
 	      signal_type sig;
@@ -840,6 +876,8 @@ namespace LDASTools
 	  assert( 0 );
 	}
       }
+      VERBOSE_DEBUGGING_30( "start_routine: return"
+			    << std::endl );
       return (Thread*)NULL;
     }
 
